#
# A Button-Bar Widget
#
# Like the Screen Manager, this is a container widget which exists
# primarily to manage the focus of subordinate widgets. This widget
# can contain any widget which obeys the "draw, set_rpos, set_lpos"
# calling convention, and which draws itself on a single line (you'll
# get a screen full of crap otherwise).
#
# To populate, create ch_button objects and "ch_buttonbar add" them
# here. It is important that the buttonbar be added to the screen
# manager before you do this (or else your buttons won't inherit
# the screen context, and you'll need to set it yourself).
#
# When this widget has input focus, the "]" and "[" characters
# choose a button.
#
# See charoo_button.tcl for more information.
#

class ch_buttonbar { 
	inherit chwidget
	
	private variable _rpos 0

	public method draw { } 
	public method onfocus { } { draw } 
	public method unfocus { } { draw } 

	public method input { ichar } 

	public constructor { rpos } {
		set _rpos $rpos
	}

}

body ch_buttonbar::draw { } {

	set s [ get_screen ] 

	$s draw_hrule [ expr $_rpos + 1 ] 

	set children [ get_children ]

	set i [ get_childcount ]
	if { $i == 0 } {
		return 0 
	}

	set tot [ $s get_cols ] 
	set per [ expr $tot / $i ] 

	set i 0

	foreach { child } $children {
		$child set_rpos $_rpos
		$child set_cpos [ expr $i * $per ] 

		$child draw

		incr i 
	}

}

body ch_buttonbar::input { ichar } { 

	set f [ get_focus ] 
	if { $f == {} } {
		return
	}

	if { $ichar == "\[" } {
		rot_focus_left
		return 0
	}

	if { $ichar == "]" } { 
		rot_focus
		return 0
	}

	return [ $f input $ichar ]
}
