#
# A text UI widget.
#
# All widgets inherit from this base class. The purpose of
# this class is to allow widgets to inherit the screen context
# of their parents, contain children, manage focus, and handle
# input events. Widgets are driven from an event loop of some
# sort, which passes a character-at-a-time to the "input" 
# "input" method of a concrete instance of a widget.
#
# A widget which receives focus has it's "onfocus" method called.
# This doesn't need to do anything if you don't want to (you'll still
# get input events), but has obvious utility when you want to visually
# show which widget has focus.
#
# A widget's "input" method returns nonzero if the widget is to
# be deleted (the actual deletion will be signaled by the invocationj
# of the object's "deleted" method).
#

proc virtual { } {
	puts "!!> PURE VIRTUAL CALLED"
	exit 1
}

class chwidget {
	public method add { wij } 
	public method del { wijid } 

	public method get_children { } 

	public method set_focus { wijid { unfocus {} } } 
	public method rot_focus { { unfocus {} } } 
	public method rot_focus_left { { unfocus {} } } 

	public method input { ichar } 	virtual
	public method onfocus { } { } 
	public method unfocus { } { } 
	public method deleted { } { }
	public method draw { } 		virtual

	private variable _nextwijid 	0
	private variable _focus		{}
	private variable _focusid	0
	private variable _children
	private variable _nchildren 	0
	private variable _screen 	{}

	public method set_screen { screen } { set _screen $screen }
	public method get_screen { } { return $_screen } 
	public method get_childcount { } { return $_nchildren } 
	public method get_focus { } { return $_focus } 
	public method get_focusid { } { return $_focusid } 

}

#
# - Increment the count of child widgets.
# - Set the focus inside of this widget to the newly-added child
# - Set the screen context of the child to our screen. 
# - Call the child's "onfocus" method
#

body chwidget::add { wij } {

	set thiswijid $_nextwijid 
	
	$wij set_screen $_screen

	set _children($thiswijid) $wij
	incr _nextwijid

	incr _nchildren

	set_focus $thiswijid

	return $thiswijid
}

#
# Remove the child, call its "delete" method, and rotate
# focus.
#

body chwidget::del { wijid } { 
	if { [ info exists _children($wijid) ] } {
		set c $_children($wijid)
		unset _children($wijid) 
		$c deleted

		if { $_focusid == $wijid } {
			rot_focus no-unfocus
		}

		incr _nchildren -1
		return 1
	}

	return 0
}

#
# Handle all focus changes (any other means of switching focus
# must be implemented in terms of this method). Call the unfocus
# method of the current focus, change the focus, and call the 
# onfocus method of the new focus.
#

body chwidget::set_focus { wijid { unfocus {} } } { 

	if { [ info exists _children($wijid) ] } {
		set f [ get_focus ] 
		if { $f != {} && $unfocus == {} } { 
			$f unfocus
		}

		set _focusid $wijid
		set _focus $_children($wijid)

		$_children($wijid) onfocus

		return 1
	}

	return 0
}

# 
# Rotate focus. Implemented in terms of set_focus.
#

body chwidget::rot_focus { { unfocus {} }  } {

	if { $_nchildren == 1 } {
		return 0
	}

	for { set i [ expr $_focusid + 1 ] } { $i < $_nextwijid } { incr i } {
		if { [ set_focus $i $unfocus ] } {
			return 1
		}
	}

	for { set i 0 } { $i < $_nextwijid } { incr i } {
		if { [ set_focus $i $unfocus ] } {
			return 1
		}
	}

	return 0
}

body chwidget::rot_focus_left { { unfocus {} }  } {

	if { $_nchildren == 1 } {
		return 0
	}

	for { set i [ expr $_focusid - 1 ] } { $i >= 0 } { incr i -1 } {
		if { [ set_focus $i $unfocus ] } {
			return 1
		}
	}

	for { set i [ expr $_focusid + 1 ] } { $i < $_nextwijid } { incr i } {
		if { [ set_focus $i $unfocus ] } {
			return 1
		}
	}

	return 0
}

body chwidget::get_children { } {

	set out {}

	foreach { name widget } [ array get _children ] {
		lappend out $widget
	}

	return $out
}
